import { put } from "redux-saga/effects";
import { app } from "./app";
import { loadingAction } from "./reducer";
import { stringifyWithMask } from "./util/json";
/**
 * A helper for ActionHandler functions (Saga)
 */
export function createActionHandlerDecorator(interceptor) {
    return (target) => {
        const descriptor = target.descriptor;
        const fn = descriptor.value;
        descriptor.value = function* (...args) {
            const rootState = app.store.getState();
            yield* interceptor(fn.bind(this, ...args), rootState);
        };
        return target;
    };
}
/**
 * A helper for regular functions
 */
export function createRegularDecorator(interceptor) {
    return (target) => {
        const descriptor = target.descriptor;
        const fn = descriptor.value;
        descriptor.value = function (...args) {
            const rootState = app.store.getState();
            interceptor(fn.bind(this, ...args), rootState);
        };
        return target;
    };
}
/**
 * To mark state.loading[identifier] during Saga execution
 */
export function Loading(identifier = "global") {
    return createActionHandlerDecorator(function* (handler) {
        try {
            yield put(loadingAction(true, identifier));
            yield* handler();
        }
        finally {
            yield put(loadingAction(false, identifier));
        }
    });
}
/**
 * To log (Result=OK) this action, including action name and parameters (masked)
 */
export function Log() {
    return (target) => {
        const descriptor = target.descriptor;
        const fn = descriptor.value;
        descriptor.value = function* (...args) {
            // Do not use fn directly, it is a different object
            const params = stringifyWithMask(app.loggerConfig && app.loggerConfig.maskedKeywords ? app.loggerConfig.maskedKeywords : [], "***", ...args);
            const actionName = descriptor.value.actionName;
            const onLogEnd = app.logger.info(actionName, params ? { params } : {});
            try {
                yield* fn.bind(this)(...args);
            }
            finally {
                onLogEnd();
            }
        };
        return target;
    };
}
/**
 * Required decorator when using lifecycle actions, including onEnter/onDestroy/...
 */
export function Lifecycle() {
    return (target) => {
        const descriptor = target.descriptor;
        descriptor.value.isLifecycle = true;
        return target;
    };
}
/**
 * Used for onTick action, to specify to tick interval in second
 */
export function Interval(second) {
    return (target) => {
        const descriptor = target.descriptor;
        descriptor.value.tickInterval = second;
        return target;
    };
}
/**
 * If specified, the Saga action cannot be entered by other threads during execution
 * Useful for error handler action
 */
export function Mutex() {
    let isLocked = false;
    return createActionHandlerDecorator(function* (handler) {
        if (!isLocked) {
            try {
                isLocked = true;
                yield* handler();
            }
            finally {
                isLocked = false;
            }
        }
    });
}
//# sourceMappingURL=decorator.js.map